@@ -19,8 +19,6 @@
 
 #include <gtk/gtk.h>
 #include <math.h>
-#include "matrix.h"
-#include "rs-batch.h"
 #include "rawstudio.h"
 #include "rawfile.h"
 #include "ciff-meta.h"
@@ -51,10 +49,16 @@ raw_crw_walker(RAWFILE *rawfile, guint offset, guint length, RS_METADATA *meta)
 		raw_get_ushort(rawfile, offset, &type);
 		raw_get_uint(rawfile, offset+2, &size);
 		raw_get_uint(rawfile, offset+6, &reloffset);
-		absoffset = blockstart + reloffset;
+		if ((type & 0xC000) == 0x4000)
+			absoffset = offset + 2;
+		else
+			absoffset = blockstart + reloffset;
 
-		switch (type)
+		switch (type & 0x3fff)
 		{
+			case 0x1029: /* FocalLength */
+				raw_get_short(rawfile, absoffset+2, &meta->focallength);
+				break;
 			case 0x102d: /* CanonCameraSettings */
 				raw_get_short(rawfile, absoffset+26, &temp); /* contrast */
 				switch(temp) 
@@ -142,6 +146,14 @@ raw_crw_walker(RAWFILE *rawfile, guint offset, guint length, RS_METADATA *meta)
 				if (wbi > 17)
 					wbi = 0;
 				break;
+			case 0x080a: /* make / model */
+				{
+					gchar makemodel[32];
+					raw_strcpy(rawfile, absoffset, makemodel, 32);
+					meta->make_ascii = g_strdup(makemodel);
+					meta->model_ascii = g_strdup(makemodel + strlen(makemodel) +1);
+				}
+				break;
 			default:
 				if (type >> 8 == 0x28 || type >> 8 == 0x30)
 					raw_crw_walker(rawfile, absoffset, size, meta);
@@ -158,11 +170,13 @@ rs_ciff_load_meta(const gchar *filename, RS_METADATA *meta)
 	guint root=0;
 	RAWFILE *rawfile;
 	rawfile = raw_open_file(filename);
+	if (!rawfile)
+		return;
 	raw_init_file_tiff(rawfile, 0);
 	if (!raw_strcmp(rawfile, 6, "HEAPCCDR", 8))
 		return;
 	raw_get_uint(rawfile, 2, &root);
-	raw_crw_walker(rawfile, root, rawfile->size-root, meta);
+	raw_crw_walker(rawfile, root, raw_get_filesize(rawfile)-root, meta);
 	raw_close_file(rawfile);
 	return;
 }
@@ -191,12 +205,14 @@ rs_ciff_load_thumb(const gchar *src)
 	}
 
 	rawfile = raw_open_file(src);
+	if (!rawfile) return(NULL);
+
 	raw_init_file_tiff(rawfile, 0);
 	if (!raw_strcmp(rawfile, 6, "HEAPCCDR", 8))
 		return(NULL);
 	raw_get_uint(rawfile, 2, &root);
 	m = rs_metadata_new();
-	raw_crw_walker(rawfile, root, rawfile->size-root, m);
+	raw_crw_walker(rawfile, root, raw_get_filesize(rawfile)-root, m);
 
 	if ((m->thumbnail_start>0) && (m->thumbnail_length>0))
 	{
