@@ -74,7 +74,7 @@ rs_conf_get_boolean(const gchar *name, gboolean *boolean_value)
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+	ret = rs_conf_get_integer(name, boolean_value);
 #endif
 	return(ret);
 }
@@ -83,11 +83,11 @@ gboolean
 rs_conf_get_boolean_with_default(const gchar *name, gboolean *boolean_value, gboolean default_value)
 {
 	gboolean ret = FALSE;
+	*boolean_value = default_value;
 #ifdef WITH_GCONF
 	GConfValue *gvalue;
 	GConfEngine *engine = get_gconf_engine();
 	GString *fullname = g_string_new(GCONF_PATH);
-	*boolean_value = default_value;
 	g_string_append(fullname, name);
 	if (engine)
 	{
@@ -105,7 +105,7 @@ rs_conf_get_boolean_with_default(const gchar *name, gboolean *boolean_value, gbo
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+	ret = rs_conf_get_integer(name, boolean_value);
 #endif
 	return(ret);
 }
@@ -123,7 +123,7 @@ rs_conf_set_boolean(const gchar *name, gboolean bool_value)
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+	ret = rs_conf_set_integer(name, bool_value);
 #endif
 	return(ret);
 }
@@ -157,11 +157,11 @@ rs_conf_get_string(const gchar *name)
 
 	if (RegOpenKeyEx( HKEY_CURRENT_USER, REGISTRY_KEY, 0, KEY_QUERY_VALUE, &hKey ) == ERROR_SUCCESS)
 	{
-	    lRet = RegQueryValueEx( hKey, CONF_LWD, NULL, NULL, NULL, &dwBufLen);
+	    lRet = RegQueryValueEx( hKey, name, NULL, NULL, NULL, &dwBufLen);
 		if (dwBufLen > 0)
 		{
 	    	szLwd = g_malloc(dwBufLen);
-	    	lRet = RegQueryValueEx( hKey, CONF_LWD, NULL, NULL, (LPBYTE) szLwd, &dwBufLen);
+	    	lRet = RegQueryValueEx( hKey, name, NULL, NULL, (LPBYTE) szLwd, &dwBufLen);
 	    	RegCloseKey( hKey );
 	    	if (lRet == ERROR_SUCCESS)
 				ret = szLwd;
@@ -229,7 +229,26 @@ rs_conf_get_integer(const gchar *name, gint *integer_value)
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+    HKEY hKey;
+    DWORD dwBufLen;
+    LONG lRet;
+
+	if (RegOpenKeyEx( HKEY_CURRENT_USER, REGISTRY_KEY, 0, KEY_QUERY_VALUE, &hKey ) == ERROR_SUCCESS)
+	{
+	    lRet = RegQueryValueEx( hKey, name, NULL, NULL, NULL, &dwBufLen);
+		if (dwBufLen > 0)
+		{
+			gint val;
+			DWORD size = sizeof(gint);
+	    	lRet = RegQueryValueEx( hKey, name, NULL, NULL, (LPBYTE) &val, &size);
+	    	RegCloseKey( hKey );
+	    	if ((lRet == ERROR_SUCCESS) && (size == sizeof(gint)))
+	    	{
+				ret = TRUE;
+				*integer_value = val;
+			}
+		}
+	}
 #endif
 	return(ret);
 }
@@ -247,7 +266,14 @@ rs_conf_set_integer(const gchar *name, const gint integer_value)
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+    HKEY hKey;
+
+	if (RegCreateKeyEx(HKEY_CURRENT_USER, REGISTRY_KEY, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, NULL) == ERROR_SUCCESS)
+	{
+		if (RegSetValueEx(hKey, name, 0, REG_DWORD, (LPBYTE) &integer_value, sizeof(gint))==ERROR_SUCCESS)
+			ret = TRUE;
+	}
+    RegCloseKey(hKey);
 #endif
 	return(ret);
 }
@@ -435,7 +461,26 @@ rs_conf_get_double(const gchar *name, gdouble *float_value)
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+    HKEY hKey;
+    DWORD dwBufLen;
+    LONG lRet;
+
+	if (RegOpenKeyEx( HKEY_CURRENT_USER, REGISTRY_KEY, 0, KEY_QUERY_VALUE, &hKey ) == ERROR_SUCCESS)
+	{
+	    lRet = RegQueryValueEx( hKey, name, NULL, NULL, NULL, &dwBufLen);
+		if (dwBufLen > 0)
+		{
+			gdouble val;
+			DWORD size = sizeof(gdouble);
+	    	lRet = RegQueryValueEx( hKey, name, NULL, NULL, (LPBYTE) &val, &size);
+	    	RegCloseKey( hKey );
+	    	if ((lRet == ERROR_SUCCESS) && (size == sizeof(gdouble)))
+	    	{
+				ret = TRUE;
+				*float_value = val;
+			}
+		}
+	}
 #endif
 	return(ret);
 }
@@ -453,7 +498,14 @@ rs_conf_set_double(const gchar *name, const gdouble float_value)
 	g_string_free(fullname, TRUE);
 #endif
 #ifdef WITH_REGISTRY
-	/* FIXME: stub */
+    HKEY hKey;
+
+	if (RegCreateKeyEx(HKEY_CURRENT_USER, REGISTRY_KEY, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, NULL) == ERROR_SUCCESS)
+	{
+		if (RegSetValueEx(hKey, name, 0, REG_BINARY, (LPBYTE) &float_value, sizeof(gdouble))==ERROR_SUCCESS)
+			ret = TRUE;
+	}
+    RegCloseKey(hKey);
 #endif
 	return(ret);
 }
@@ -470,9 +522,8 @@ rs_conf_get_list_string(const gchar *name)
 	if (engine)
 		list = gconf_engine_get_list(engine, fullname->str, GCONF_VALUE_STRING, NULL);
 	g_string_free(fullname, TRUE);
-#endif
-#ifdef WITH_REGISTRY
-	/* FIXME: stub */
+#else
+	/* FIXME: windows stub */
 #endif
 	return list;
 }
@@ -489,9 +540,8 @@ rs_conf_set_list_string(const gchar *name, GSList *list)
 	if (engine)
 		ret = gconf_engine_set_list(engine, fullname->str, GCONF_VALUE_STRING, list, NULL);
 	g_string_free(fullname, TRUE);
-#endif
-#ifdef WITH_REGISTRY
-	/* FIXME: stub */
+#else
+	/* FIXME: windows stub */
 #endif
 	return(ret);
 }
