@@ -52,7 +52,19 @@
 #define CONF_ROI_GRID "roi_grid"
 #define CONF_CROP_ASPECT "crop_aspect"
 #define CONF_SHOW_FILENAMES "show_filenames_in_iconview"
-
+#define CONF_FULLSCREEN "fullscreen"
+#define CONF_SHOW_TOOLBOX_FULLSCREEN "show_toolbox_fullscreen"
+#define CONF_SHOW_TOOLBOX "show_toolbox"
+#define CONF_SHOW_ICONBOX_FULLSCREEN "show_iconbox_fullscreen"
+#define CONF_SHOW_ICONBOX "show_iconbox"
+#define CONF_SHOW_TOOLBOX_EXPOSURE "show_toolbox_exposure"
+#define CONF_SHOW_TOOLBOX_SATURATION "show_toolbox_saturation"
+#define CONF_SHOW_TOOLBOX_HUE "show_toolbox_hue"
+#define CONF_SHOW_TOOLBOX_CONTRAST "show_toolbox_contrast"
+#define CONF_SHOW_TOOLBOX_WARMTH "show_toolbox_warmth"
+#define CONF_SHOW_TOOLBOX_CURVE "show_toolbox_curve"
+#define CONF_SHOW_TOOLBOX_TRANSFORM "show_toolbox_transform"
+#define CONF_SHOW_TOOLBOX_HIST "show_toolbox_hist"
 
 #define DEFAULT_CONF_EXPORT_DIRECTORY "exports/"
 #define DEFAULT_CONF_EXPORT_FILENAME "%f_%2c"
@@ -62,6 +74,19 @@
 #define DEFAULT_CONF_BATCH_FILENAME "%f_%2c"
 #define DEFAULT_CONF_BATCH_FILETYPE "jpeg"
 #define DEFAULT_CONF_BATCH_JPEG_QUALITY "100"
+#define DEFAULT_CONF_FULLSCREEN FALSE
+#define DEFAULT_CONF_SHOW_TOOLBOX_FULLSCREEN TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX TRUE
+#define DEFAULT_CONF_SHOW_ICONBOX_FULLSCREEN FALSE
+#define DEFAULT_CONF_SHOW_ICONBOX TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_EXPOSURE TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_SATURATION TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_HUE TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_CONTRAST TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_WARMTH TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_CURVE TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_TRANSFORM TRUE
+#define DEFAULT_CONF_SHOW_TOOLBOX_HIST TRUE
 
 
 /* get the last working directory from gconf */
