@@ -18,7 +18,9 @@
  */
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
+#include <config.h>
 #include "rawstudio.h"
 #include "conf_interface.h"
 #include "gtk-interface.h"
@@ -28,6 +30,15 @@
 #include <gettext.h>
 #include <lcms.h>
 
+struct _RS_CONFBOX
+{
+	GtkWidget *widget;
+	GtkListStore *model;
+	const gchar *conf_key;
+	gpointer user_data;
+	void (*callback)(gpointer active, gpointer user_data);
+};
+
 static void gui_confbox_changed(GtkComboBox *filetype_combo, gpointer callback_data);
 static gboolean gui_confbox_deleted(GtkWidget *widget, GdkEvent *event, gpointer callback_data);
 static void gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
@@ -39,6 +50,8 @@ static void gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_d
 static void gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data);
 static void spinner_set_conf(GtkSpinButton *spinner, gpointer user_data);
 static void slider_with_spinner_set_conf(GtkSpinButton *spinner, gpointer user_data);
+static gboolean rs_gtk_tree_model_count_helper(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
+static gint rs_gtk_tree_model_count(GtkTreeModel *model);
 
 static const gchar *color_profiles[] = {
 	"*.icc", 
@@ -424,7 +437,13 @@ gui_cms_choose_profile(const gchar *path)
 	if (path)
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fc), path);
 	else
-		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fc), "/usr/share/color/icc");
+	{
+#define DEAULT_PROFILE_PATH "/usr/share/color/icc"
+		if (g_access(DEAULT_PROFILE_PATH, X_OK)==0)
+			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fc),
+				DEAULT_PROFILE_PATH);
+#undef DEAULT_PROFILE_PATH
+	}
 
 	file_filter_all = gtk_file_filter_new();
 	file_filter_color_profiles = gtk_file_filter_new();
@@ -454,7 +473,7 @@ gui_cms_choose_profile(const gchar *path)
 		{
 			GtkWidget *warning = gui_dialog_make_from_text(GTK_STOCK_DIALOG_WARNING,
 				_("Not a valid color profile."), 
-				_("The file you selected does not appear to be a valid color profile."));
+				_("The file you selected does not appear to be a valid color profile for this version of Rawstudio."));
 			GtkWidget *ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
 			gtk_dialog_add_action_widget(GTK_DIALOG(warning), ok_button, GTK_RESPONSE_ACCEPT);
 			
@@ -479,10 +498,11 @@ gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data)
 
 	if (filename)
 	{
+		GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));
 		gtk_combo_box_append_text(GTK_COMBO_BOX(combobox), g_basename(filename));
-		// FIXME: gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), __SELECT_LAST__);
 		rs_conf_add_string_to_list_string(CONF_CMS_IN_PROFILE_LIST, filename);
 		rs_conf_set_integer(CONF_CMS_IN_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
+		gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), rs_gtk_tree_model_count(model)-1);
 	}
 	return;
 }
@@ -497,10 +517,11 @@ gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data)
 
 	if (filename)
 	{
+		GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));
 		gtk_combo_box_append_text(GTK_COMBO_BOX(combobox), g_basename(filename));
-		// FIXME: gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), __SELECT_LAST__);
 		rs_conf_add_string_to_list_string(CONF_CMS_DI_PROFILE_LIST, filename);
 		rs_conf_set_integer(CONF_CMS_DI_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
+		gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), rs_gtk_tree_model_count(model)-1);
 	}
 	return;
 }
@@ -515,10 +536,11 @@ gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data)
 
 	if (filename)
 	{
+		GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));
 		gtk_combo_box_append_text(GTK_COMBO_BOX(combobox), g_basename(filename));
-		// FIXME: gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), __SELECT_LAST__);
 		rs_conf_add_string_to_list_string(CONF_CMS_EX_PROFILE_LIST, filename);
 		rs_conf_set_integer(CONF_CMS_EX_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
+		gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), rs_gtk_tree_model_count(model)-1);
 	}
 	return;
 }
@@ -563,7 +585,7 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 	cms_in_profile_hbox = gtk_hbox_new(FALSE, 0);
 	cms_in_profile_label = gtk_label_new(_("Input profile"));
 	gtk_misc_set_alignment(GTK_MISC(cms_in_profile_label), 0.0, 0.5);
-	cms_in_profile_combobox = gtk_combo_box_entry_new_text();
+	cms_in_profile_combobox = gtk_combo_box_new_text();
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cms_in_profile_combobox), _("BuiltInRGBProfile"));
 	temp_conf_gslist = rs_conf_get_list_string(CONF_CMS_IN_PROFILE_LIST);
 	temp_conf_gint = 0;
@@ -602,7 +624,7 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 	cms_di_profile_hbox = gtk_hbox_new(FALSE, 0);
 	cms_di_profile_label = gtk_label_new(_("Display profile"));
 	gtk_misc_set_alignment(GTK_MISC(cms_di_profile_label), 0.0, 0.5);
-	cms_di_profile_combobox = gtk_combo_box_entry_new_text();
+	cms_di_profile_combobox = gtk_combo_box_new_text();
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cms_di_profile_combobox), _("sRGB"));
 	temp_conf_gslist = rs_conf_get_list_string(CONF_CMS_DI_PROFILE_LIST);
 	temp_conf_gint = 0;
@@ -641,7 +663,7 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 	cms_ex_profile_hbox = gtk_hbox_new(FALSE, 0);
 	cms_ex_profile_label = gtk_label_new(_("Export profile"));
 	gtk_misc_set_alignment(GTK_MISC(cms_ex_profile_label), 0.0, 0.5);
-	cms_ex_profile_combobox = gtk_combo_box_entry_new_text();
+	cms_ex_profile_combobox = gtk_combo_box_new_text();
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cms_ex_profile_combobox), _("sRGB"));
 	temp_conf_gslist = rs_conf_get_list_string(CONF_CMS_EX_PROFILE_LIST);
 	temp_conf_gint = 0;
@@ -794,14 +816,14 @@ window_key_press_event (GtkWidget   *widget,
    * before the accelerator activation scheme.
    */
 
-  /* text widgets get all key events first */
-  if (G_UNLIKELY (GTK_IS_EDITABLE (focus) || GTK_IS_TEXT_VIEW (focus)))
-    handled = gtk_window_propagate_key_event (window, event);
-
-  /* invoke control/alt accelerators */
+  /* control/alt accelerators get all key events first */
   if (! handled && event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
     handled = gtk_window_activate_key (window, event);
 
+  /* invoke text widgets  */
+  if (G_UNLIKELY (GTK_IS_EDITABLE (focus) || GTK_IS_TEXT_VIEW (focus)))
+    handled = gtk_window_propagate_key_event (window, event);
+
   /* invoke focus widget handlers */
   if (! handled)
     handled = gtk_window_propagate_key_event (window, event);
@@ -817,9 +839,10 @@ window_key_press_event (GtkWidget   *widget,
   return handled;
 }
 
-/* Should be used like this:
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, pos_menu_below_widget, widget, 0, GDK_CURRENT_TIME);
-*/
+/**
+ * Function to help gtk_menu_popup(), positiones the popup menu below a widget
+ * Should be used like this: gtk_menu_popup(GTK_MENU(menu), NULL, NULL, pos_menu_below_widget, widget, 0, GDK_CURRENT_TIME);
+ */
 void
 pos_menu_below_widget (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data)
 {
@@ -834,6 +857,13 @@ pos_menu_below_widget (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpoin
 	return;
 }
 
+/**
+ * Put a GtkFrame around a widget
+ * @param widget The widget to frame
+ * @param title Title for the frame
+ * @param shadowtype A GtkShadowType
+ * @return A new GtkFrame
+ */
 GtkWidget *
 gui_framed(GtkWidget *widget, const gchar *title, GtkShadowType shadowtype)
 {
@@ -845,3 +875,18 @@ gui_framed(GtkWidget *widget, const gchar *title, GtkShadowType shadowtype)
 
 	return(frame);
 }
+
+static gboolean
+rs_gtk_tree_model_count_helper(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+{
+	(*((gint *)data))++;
+	return FALSE;
+}
+
+static gint
+rs_gtk_tree_model_count(GtkTreeModel *model)
+{
+	gint n = 0;
+	gtk_tree_model_foreach (model, rs_gtk_tree_model_count_helper, &n);
+	return n;
+}
