@@ -20,24 +20,6 @@
 #ifndef GTK_INTERFACE_H
 #define GTK_INTERFACE_H
 
-enum { 
-	PIXBUF_COLUMN,
-	TEXT_COLUMN,
-	DATA_COLUMN,
-	FULLNAME_COLUMN,
-	PRIORITY_COLUMN,
-	NUM_COLUMNS
-};
-
-enum {
-	PRIO_U = 0,
-	PRIO_D = 51,
-	PRIO_1 = 1,
-	PRIO_2 = 2,
-	PRIO_3 = 3,
-	PRIO_ALL = 255
-};
-
 enum {
 	OP_NONE = 0,
 	OP_BUSY,
@@ -55,6 +37,7 @@ extern void update_histogram(RS_BLOB *rs);
 extern gboolean update_preview_callback(GtkAdjustment *caller, RS_BLOB *rs);
 extern gboolean update_previewtable_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs);
 extern gboolean update_scale_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs);
+extern void icon_set_flags(const gchar *filename, GtkTreeIter *iter, const guint *priority, const gboolean *exported);
 extern void gui_dialog_simple(gchar *title, gchar *message);
 extern GtkWidget *gui_dialog_make_from_text(const gchar *stock_id, gchar *primary_text, gchar *secondary_text);
 extern void gui_set_values(RS_BLOB *rs, gint x, gint y);
