@@ -16,20 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-
 #define ENDIANSWAP4(a) (((a) & 0x000000FF) << 24 | ((a) & 0x0000FF00) << 8 | ((a) & 0x00FF0000) >> 8) | (((a) & 0xFF000000) >> 24)
 #define ENDIANSWAP2(a) (((a) & 0x00FF) << 8) | (((a) & 0xFF00) >> 8)
 
-#define raw_fgetc(rawfile, pos) (*(guchar *)((rawfile)->map+((pos)++)))
-
-typedef struct _rawfile {
-	gint fd;
-	guint size;
-	void *map;
-	gushort byteorder;
-	guint first_ifd_offset;
-	guint base;
-} RAWFILE;
+typedef struct _RAWFILE RAWFILE; /* Defined in rawfile.c */
 
 void raw_init();
 RAWFILE *raw_open_file(const gchar *filename);
@@ -47,3 +37,9 @@ gchar *raw_strdup(RAWFILE *rawfile, guint pos, gint len);
 GdkPixbuf *raw_get_pixbuf(RAWFILE *rawfile, guint pos, guint length);
 void raw_close_file(RAWFILE *rawfile);
 void raw_reset_base(RAWFILE *rawfile);
+gint raw_get_base(RAWFILE *rawfile);
+gushort raw_get_byteorder(RAWFILE *rawfile);
+void raw_set_byteorder(RAWFILE *rawfile, gushort byteorder);
+guint get_first_ifd_offset(RAWFILE *rawfile);
+void *raw_get_map(RAWFILE *rawfile);
+guint raw_get_filesize(RAWFILE *rawfile);
