@@ -46,13 +46,10 @@
 #include "rs-arch.h"
 #include "rs-batch.h"
 #include "rs-cms.h"
+#include "rs-store.h"
 
 static void update_scaled(RS_BLOB *rs, gboolean force);
-static inline void rs_render_mask(guchar *pixels, guchar *mask, guint length);
 static gboolean rs_render_idle(RS_BLOB *rs);
-static void rs_render_overlay(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride,
-	guchar *mask, gint mask_rowstride, RS_CMS *cms);
 static RS_SETTINGS *rs_settings_new();
 static RS_SETTINGS_DOUBLE *rs_settings_double_new();
 static void rs_settings_double_free(RS_SETTINGS_DOUBLE *rssd);
@@ -66,6 +63,12 @@ static void rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quart
 
 RS_FILETYPE *filetypes;
 
+GdkPixbuf *icon_priority_1;
+GdkPixbuf *icon_priority_2;
+GdkPixbuf *icon_priority_3;
+GdkPixbuf *icon_priority_D;
+GdkPixbuf *icon_exported;
+
 static void
 rs_add_filetype(gchar *id, gint filetype, const gchar *ext, gchar *description,
 	RS_PHOTO *(*load)(const gchar *),
@@ -98,35 +101,37 @@ static void
 rs_init_filetypes(void)
 {
 	filetypes = NULL;
-	rs_add_filetype("cr2", FILETYPE_RAW, "cr2", _("Canon CR2"),
+	rs_add_filetype("cr2", FILETYPE_RAW, ".cr2", _("Canon CR2"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
-	rs_add_filetype("crw", FILETYPE_RAW, "crw", _("Canon CIFF"),
+	rs_add_filetype("crw", FILETYPE_RAW, ".crw", _("Canon CIFF"),
 		rs_photo_open_dcraw, rs_ciff_load_thumb, rs_ciff_load_meta, NULL);
-	rs_add_filetype("nef", FILETYPE_RAW, "nef", _("Nikon NEF"),
+	rs_add_filetype("nef", FILETYPE_RAW, ".nef", _("Nikon NEF"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
-	rs_add_filetype("mrw", FILETYPE_RAW, "mrw", _("Minolta raw"),
+	rs_add_filetype("mrw", FILETYPE_RAW, ".mrw", _("Minolta raw"),
 		rs_photo_open_dcraw, rs_mrw_load_thumb, rs_mrw_load_meta, NULL);
-	rs_add_filetype("cr-tiff", FILETYPE_RAW, "tif", _("Canon TIFF"),
+	rs_add_filetype("cr-tiff", FILETYPE_RAW, ".tif", _("Canon TIFF"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
-	rs_add_filetype("arw", FILETYPE_RAW, "arw", _("Sony"),
+	rs_add_filetype("arw", FILETYPE_RAW, ".arw", _("Sony"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
-	rs_add_filetype("kdc", FILETYPE_RAW, "kdc", _("Kodak"),
+	rs_add_filetype("kdc", FILETYPE_RAW, ".kdc", _("Kodak"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
-	rs_add_filetype("x3f", FILETYPE_RAW, "x3f", _("Sigma"),
+	rs_add_filetype("x3f", FILETYPE_RAW, ".x3f", _("Sigma"),
 		rs_photo_open_dcraw, rs_x3f_load_thumb, NULL, NULL);
-	rs_add_filetype("orf", FILETYPE_RAW, "orf", "",
+	rs_add_filetype("orf", FILETYPE_RAW, ".orf", "",
 		rs_photo_open_dcraw, rs_tiff_load_thumb, NULL, NULL);
-	rs_add_filetype("raw", FILETYPE_RAW, "raw", _("Panasonic raw"),
+	rs_add_filetype("raw", FILETYPE_RAW, ".raw", _("Panasonic raw"),
 		rs_photo_open_dcraw, rs_panasonic_load_thumb, rs_panasonic_load_meta, NULL);
-	rs_add_filetype("pef", FILETYPE_RAW, "pef", _("Pentax raw"),
+	rs_add_filetype("pef", FILETYPE_RAW, ".pef", _("Pentax raw"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, NULL, NULL);
-	rs_add_filetype("jpeg", FILETYPE_JPEG, "jpg", _("JPEG (Joint Photographic Experts Group)"),
+	rs_add_filetype("dng", FILETYPE_RAW, "dng", _("Adobe Digital negative"),
+		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+	rs_add_filetype("jpeg", FILETYPE_JPEG, ".jpg", _("JPEG (Joint Photographic Experts Group)"),
 		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
-	rs_add_filetype("png", FILETYPE_PNG, "png", _("PNG (Portable Network Graphics)"),
+	rs_add_filetype("png", FILETYPE_PNG, ".png", _("PNG (Portable Network Graphics)"),
 		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
-	rs_add_filetype("tiff8", FILETYPE_TIFF8, "tif", _("8-bit TIFF (Tagged Image File Format)"),
+	rs_add_filetype("tiff8", FILETYPE_TIFF8, ".tif", _("8-bit TIFF (Tagged Image File Format)"),
 		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
-	rs_add_filetype("tiff16", FILETYPE_TIFF16, "tif", _("16-bit TIFF (Tagged Image File Format)"),
+	rs_add_filetype("tiff16", FILETYPE_TIFF16, ".tif", _("16-bit TIFF (Tagged Image File Format)"),
 		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
 	return;
 }
@@ -150,14 +155,13 @@ update_scaled(RS_BLOB *rs, gboolean force)
 		rs->photo->scaled = rs_image16_transform(rs->photo->input, NULL,
 			&rs->photo->affine, &rs->photo->inverse_affine, rs->photo->crop,
 			rs->preview_width-12, rs->preview_height-12, TRUE,
-			scale, rs->photo->angle, rs->photo->orientation);
+			scale, rs->photo->angle, rs->photo->orientation, NULL);
 		matrix3_affine_transform_point_int(&rs->photo->affine,
 			rs->roi.x1, rs->roi.y1,
 			&rs->roi_scaled.x1, &rs->roi_scaled.y1);
 		matrix3_affine_transform_point_int(&rs->photo->affine,
 			rs->roi.x2, rs->roi.y2,
 			&rs->roi_scaled.x2, &rs->roi_scaled.y2);
-		rs->preview_done = TRUE; /* stop rs_render_idle() */
 	}
 
 	/* allocate 8 bit buffers if needed */
@@ -168,12 +172,6 @@ update_scaled(RS_BLOB *rs, gboolean force)
 		gtk_widget_set_size_request(rs->preview_drawingarea, rs->photo->scaled->w, rs->photo->scaled->h);
 	}
 
-	/* allocate mask-buffer if needed */
-	if (!rs_image16_8_cmp_size(rs->photo->scaled, rs->photo->mask))
-	{
-		rs_image8_free(rs->photo->mask);
-		rs->photo->mask = rs_image8_new(rs->photo->scaled->w, rs->photo->scaled->h, 1, 1);
-	}
 	return;
 }
 
@@ -195,7 +193,6 @@ rs_photo_prepare(RS_PHOTO *photo)
 
 	matrix4_color_saturate(&photo->mat, photo->settings[photo->current_setting]->saturation);
 	matrix4_color_hue(&photo->mat, photo->settings[photo->current_setting]->hue);
-	matrix4_to_matrix4int(&photo->mat, &photo->mati);
 }
 
 void
@@ -203,8 +200,10 @@ update_preview(RS_BLOB *rs, gboolean update_table, gboolean update_scale)
 {
 	if(unlikely(!rs->photo)) return;
 
+	rs_render_idle_stop(rs);
 	if (update_table)
-		rs_render_previewtable(rs->photo->settings[rs->photo->current_setting]->contrast);
+		rs_render_previewtable(rs->photo->settings[rs->photo->current_setting]->contrast,
+			rs->photo->settings[rs->photo->current_setting]->curve_samples, rs->previewtable8, rs->previewtable16);
 	update_scaled(rs, update_scale);
 	rs_photo_prepare(rs->photo);
 	update_preview_region(rs, rs->preview_exposed, TRUE);
@@ -213,7 +212,7 @@ update_preview(RS_BLOB *rs, gboolean update_table, gboolean update_scale)
 	if (GTK_WIDGET_VISIBLE(rs->histogram_image))
 	{
 		memset(rs->histogram_table, 0x00, sizeof(guint)*3*256);
-		rs_render_histogram_table(rs->photo, rs->histogram_dataset, (guint *) rs->histogram_table);
+		rs_render_histogram_table(&rs->photo->mat, rs->photo->pre_mul, rs->previewtable8, rs->histogram_dataset, (guint *) rs->histogram_table);
 		update_histogram(rs);
 	}
 
@@ -251,25 +250,18 @@ update_preview_region(RS_BLOB *rs, RS_RECT *region, gboolean force_render)
 	if (unlikely(region->x2 < region->x1))
 		return;
 
-	pixels = rs->photo->preview->pixels+(region->y1*rs->photo->preview->rowstride
-		+ region->x1*rs->photo->preview->pixelsize);
-	in = rs->photo->scaled->pixels+(region->y1*rs->photo->scaled->rowstride
-		+ region->x1*rs->photo->scaled->pixelsize);
+	pixels = GET_PIXEL(rs->photo->preview, region->x1, region->y1);
+	in = GET_PIXEL(rs->photo->scaled, region->x1, region->y1);
 
 	if (likely((!rs->preview_done) || force_render))
 	{
+		rs_render(&rs->photo->mat, rs->photo->pre_mul,
+			rs->previewtable8, rs->previewtable16,
+			w, h, in, rs->photo->scaled->rowstride,
+			pixels, rs->photo->preview->rowstride,
+			rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
 		if (unlikely(rs->show_exposure_overlay))
-		{
-			guchar *mask = rs->photo->mask->pixels+(region->y1*rs->photo->mask->rowstride
-				+region->x1*rs->photo->mask->pixelsize);
-			rs_render_overlay(rs->photo, w, h, in, rs->photo->scaled->rowstride,
-				rs->photo->scaled->pixelsize, pixels, rs->photo->preview->rowstride,
-				mask, rs->photo->mask->rowstride, rs->cms);
-		}
-		else
-			rs_render(rs->photo, w, h, in, rs->photo->scaled->rowstride,
-				rs->photo->scaled->pixelsize, pixels, rs->photo->preview->rowstride,
-				rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
+			rs_image8_render_exposure_mask(rs->photo->preview, -1);
 
 		if (unlikely(rs->mark_roi))
 		{
@@ -321,8 +313,7 @@ update_preview_region(RS_BLOB *rs, RS_RECT *region, gboolean force_render)
 		pango_layout_set_text(text_layout, text->str, -1);
 		pango_layout_get_pixel_size(text_layout, &text_width, &text_height);
 
-		pixels = rs->photo->preview->pixels+(rs->roi_scaled.y1*rs->photo->preview->rowstride
-			+ rs->roi_scaled.x1*rs->photo->preview->pixelsize);
+		pixels = GET_PIXEL(rs->photo->preview, rs->roi_scaled.x1, rs->roi_scaled.y1);
 		/* draw all our stuff to blit-buffer */
 		gdk_draw_drawable(blitter, gc, /* not ROI */
 			rs->preview_backing_notroi,
@@ -490,25 +481,11 @@ update_preview_region(RS_BLOB *rs, RS_RECT *region, gboolean force_render)
 	return;
 }
 
-inline void
-rs_render_mask(guchar *pixels, guchar *mask, guint length)
+void
+rs_render_idle_stop(RS_BLOB *rs)
 {
-	guchar *pixel = pixels;
-	while(length--)
-	{
-		*mask = 0x0;
-		if (pixel[R] == 255)
-			*mask |= MASK_OVER;
-		else if (pixel[G] == 255)
-			*mask |= MASK_OVER;
-		else if (pixel[B] == 255)
-			*mask |= MASK_OVER;
-		else if ((pixel[R] < 2 && pixel[G] < 2) && pixel[B] < 2)
-			*mask |= MASK_UNDER;
-		pixel+=3;
-		mask++;
-	}
-	return;
+	rs->preview_done = TRUE; /* stop rs_render_idle() */
+	rs_render_idle(rs);
 }
 
 static gboolean
@@ -516,7 +493,7 @@ rs_render_idle(RS_BLOB *rs)
 {
 	gint row;
 	gushort *in;
-	guchar *out, *mask;
+	guchar *out;
 
 	if (rs->in_use && (!rs->preview_done))
 		for(row=rs->preview_idle_render_lastrow; row<rs->photo->scaled->h; row++)
@@ -524,16 +501,12 @@ rs_render_idle(RS_BLOB *rs)
 			in = rs->photo->scaled->pixels + row*rs->photo->scaled->rowstride;
 			out = rs->photo->preview->pixels + row*rs->photo->preview->rowstride;
 
+			rs_render(&rs->photo->mat, rs->photo->pre_mul,
+				rs->previewtable8, rs->previewtable16,
+				rs->photo->scaled->w, 1, in, rs->photo->scaled->rowstride,
+				out, rs->photo->preview->rowstride, rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
 			if (unlikely(rs->show_exposure_overlay))
-			{
-				mask = rs->photo->mask->pixels + row*rs->photo->mask->rowstride;
-				rs_render_overlay(rs->photo, rs->photo->scaled->w, 1, in, rs->photo->scaled->rowstride,
-					rs->photo->scaled->pixelsize, out, rs->photo->preview->rowstride,
-					mask, rs->photo->mask->rowstride, rs->cms);
-			}
-			else
-				rs_render(rs->photo, rs->photo->scaled->w, 1, in, rs->photo->scaled->rowstride,
-					rs->photo->scaled->pixelsize, out, rs->photo->preview->rowstride, rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
+				rs_image8_render_exposure_mask(rs->photo->preview, row);
 	
 			gdk_draw_rgb_image(rs->preview_backing,
 				rs->preview_drawingarea->style->fg_gc[GTK_STATE_NORMAL], 0, row,
@@ -563,40 +536,6 @@ rs_render_idle(RS_BLOB *rs)
 	return(FALSE);
 }
 
-static void
-rs_render_overlay(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride,
-	guchar *mask, gint mask_rowstride, RS_CMS *cms)
-{
-	gint y,x;
-	gint maskoffset, destoffset;
-	rs_render(photo, width, height, in, in_rowstride, in_channels, out, out_rowstride, rs_cms_get_transform(cms, TRANSFORM_DISPLAY));
-	for(y=0 ; y<height ; y++)
-	{
-		destoffset = y * out_rowstride;
-		maskoffset = y * mask_rowstride;
-		rs_render_mask(out+destoffset, mask+maskoffset, width);
-		for(x=0 ; x<width ; x++)
-		{
-			if (mask[maskoffset] & MASK_OVER)
-			{
-				out[destoffset+R] = 255;
-				out[destoffset+B] = 0;
-				out[destoffset+G] = 0;
-			}
-			if (mask[maskoffset] & MASK_UNDER)
-			{
-				out[destoffset+R] = 0;
-				out[destoffset+B] = 255;
-				out[destoffset+G] = 0;
-			}
-			maskoffset++;
-			destoffset+=3;
-		}
-	}
-	return;
-}
-
 void
 rs_reset(RS_BLOB *rs)
 {
@@ -636,6 +575,8 @@ rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *
 	rs_settings_double->contrast = GETVAL(rs_settings->contrast);
 	rs_settings_double->warmth = GETVAL(rs_settings->warmth);
 	rs_settings_double->tint = GETVAL(rs_settings->tint);
+	rs_settings_double->curve_samples = rs_settings->curve_samples;
+	rs_curve_widget_get_knots(RS_CURVE_WIDGET(rs_settings->curve), &rs_settings_double->curve_knots, &rs_settings_double->curve_nknots);
 	return;
 }
 
@@ -652,6 +593,19 @@ rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SET
 	SETVAL(rs_settings->contrast, rs_settings_double->contrast);
 	SETVAL(rs_settings->warmth, rs_settings_double->warmth);
 	SETVAL(rs_settings->tint, rs_settings_double->tint);
+	rs_curve_widget_reset(RS_CURVE_WIDGET(rs_settings->curve));
+	if (rs_settings_double->curve_nknots>0)
+	{
+		gint i;
+		for(i=0;i<rs_settings_double->curve_nknots;i++)
+			rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), rs_settings_double->curve_knots[i*2+0],
+				rs_settings_double->curve_knots[i*2+1]);
+	}
+	else
+	{
+		rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), 0.0f, 0.0f);
+		rs_curve_widget_add_knot(RS_CURVE_WIDGET(rs_settings->curve), 1.0f, 1.0f);
+	}
 	return;
 }
 
@@ -670,12 +624,19 @@ rs_settings_reset(RS_SETTINGS *rss, guint mask)
 		gtk_adjustment_set_value((GtkAdjustment *) rss->warmth, 0.0);
 	if (mask & MASK_TINT)
 		gtk_adjustment_set_value((GtkAdjustment *) rss->tint, 0.0);
+	if (mask & MASK_CURVE)
+	{
+		rs_curve_widget_reset(RS_CURVE_WIDGET(rss->curve));
+		rs_curve_widget_add_knot(RS_CURVE_WIDGET(rss->curve), 0.0f, 0.0f);
+		rs_curve_widget_add_knot(RS_CURVE_WIDGET(rss->curve), 1.0f, 1.0f);
+	}
 	return;
 }
 
 static RS_SETTINGS *
 rs_settings_new(void)
 {
+	gint i;
 	RS_SETTINGS *rss;
 	rss = g_malloc(sizeof(RS_SETTINGS));
 	rss->exposure = gtk_adjustment_new(0.0, -3.0, 3.0, 0.1, 0.5, 0.0);
@@ -684,6 +645,10 @@ rs_settings_new(void)
 	rss->contrast = gtk_adjustment_new(1.0, 0.0, 3.0, 0.1, 0.5, 0.0);
 	rss->warmth = gtk_adjustment_new(0.0, -2.0, 2.0, 0.1, 0.5, 0.0);
 	rss->tint = gtk_adjustment_new(0.0, -2.0, 2.0, 0.1, 0.5, 0.0);
+	rss->curve = rs_curve_widget_new();
+	rs_curve_widget_set_array(RS_CURVE_WIDGET(rss->curve), rss->curve_samples, 65536);
+	for (i=0;i<65536;i++)
+		rss->curve_samples[i] = ((gfloat) i)/65535.0f;
 	return(rss);
 }
 
@@ -698,9 +663,48 @@ static RS_SETTINGS_DOUBLE
 	rssd->contrast = 1.0;
 	rssd->warmth = 0.0;
 	rssd->tint = 0.0;
+	rssd->curve_samples = NULL;
+	rssd->curve_nknots = 0;
+	rssd->curve_knots = NULL;
 	return rssd;
 }
 
+void
+rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask)
+{
+	g_assert(in);
+	g_assert(out);
+	if (mask & MASK_EXPOSURE)
+		out->exposure = in->exposure;
+	if (mask & MASK_SATURATION)
+		out->saturation = in->saturation;
+	if (mask & MASK_HUE)
+		out->hue = in->hue;
+	if (mask & MASK_CONTRAST)
+		out->contrast = in->contrast;
+	if (mask & MASK_WARMTH)
+		out->warmth = in->warmth;
+	if (mask & MASK_TINT)
+		out->tint = in->tint;
+	if (mask & MASK_CURVE)
+	{
+
+		if (in->curve_nknots>1)
+		{
+			gint i;
+
+			out->curve_nknots = in->curve_nknots;
+			if (out->curve_knots)
+				g_free(out->curve_knots);
+			out->curve_knots = g_new(gfloat, out->curve_nknots*2);
+			
+			for(i=0;i<in->curve_nknots*2;i++)
+				out->curve_knots[i] = in->curve_knots[i];
+		}
+	}
+	return;
+}
+
 static void
 rs_settings_double_free(RS_SETTINGS_DOUBLE *rssd)
 {
@@ -712,6 +716,7 @@ RS_METADATA *
 rs_metadata_new(void)
 {
 	RS_METADATA *metadata;
+	gint i;
 	metadata = g_malloc(sizeof(RS_METADATA));
 	metadata->make = MAKE_UNKNOWN;
 	metadata->make_ascii = NULL;
@@ -724,9 +729,17 @@ rs_metadata_new(void)
 	metadata->thumbnail_length = 0;
 	metadata->preview_start = 0;
 	metadata->preview_length = 0;
+	metadata->preview_planar_config = 0;
+	metadata->preview_width = 0;
+	metadata->preview_height = 0;
 	metadata->cam_mul[0] = -1.0;
 	metadata->contrast = -1.0;
 	metadata->saturation = -1.0;
+	metadata->sharpness = -1.0;
+	metadata->color_tone = -1.0;
+	metadata->focallength = -1;
+	for(i=0;i<4;i++)
+		metadata->cam_mul[i] = 1.0f;
 	matrix4_identity(&metadata->adobe_coeff);
 	metadata->data = NULL;
 	return(metadata);
@@ -770,7 +783,6 @@ rs_photo_new()
 	photo->input = NULL;
 	photo->scaled = NULL;
 	photo->preview = NULL;
-	photo->mask = NULL;
 	ORIENTATION_RESET(photo->orientation);
 	photo->current_setting = 0;
 	photo->priority = PRIO_U;
@@ -779,6 +791,7 @@ rs_photo_new()
 		photo->settings[c] = rs_settings_double_new();
 	photo->crop = NULL;
 	photo->angle = 0.0;
+	photo->exported = FALSE;
 	matrix3_identity(&photo->inverse_affine);
 	return(photo);
 }
@@ -836,20 +849,23 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 	RS_IMAGE16 *image16;
 	gint quality = 100;
 	gboolean uncompressed_tiff = FALSE;
+	guchar table8[65536];
+	gushort table16[65536];
 
 	/* transform and crop */
 	rsi = rs_image16_transform(photo->input, NULL,
 			NULL, NULL, photo->crop, width, height, FALSE, scale,
-			photo->angle, photo->orientation);
+			photo->angle, photo->orientation, NULL);
 
+	rs_render_previewtable(photo->settings[photo->current_setting]->contrast,
+		photo->settings[photo->current_setting]->curve_samples, table8, table16);
 	/* actually save */
 	switch (filetype)
 	{
 		case FILETYPE_JPEG:
 			image8 = rs_image8_new(rsi->w, rsi->h, 3, 3);
-			rs_render(photo, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, rsi->channels,
-				image8->pixels, image8->rowstride,
+			rs_render(&photo->mat, photo->pre_mul, table8, table16, rsi->w, rsi->h, rsi->pixels,
+				rsi->rowstride, image8->pixels, image8->rowstride,
 				rs_cms_get_transform(cms, TRANSFORM_EXPORT));
 
 			rs_conf_get_integer(CONF_EXPORT_JPEG_QUALITY, &quality);
@@ -863,8 +879,7 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 			break;
 		case FILETYPE_PNG:
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
-			rs_render(photo, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, rsi->channels,
+			rs_render(&photo->mat, photo->pre_mul, table8, table16, rsi->w, rsi->h, rsi->pixels, rsi->rowstride,
 				gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf),
 				rs_cms_get_transform(cms, TRANSFORM_EXPORT));
 			gdk_pixbuf_save(pixbuf, filename, "png", NULL, NULL);
@@ -873,9 +888,8 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 		case FILETYPE_TIFF8:
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
 			image8 = rs_image8_new(rsi->w, rsi->h, 3, 3);
-			rs_render(photo, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, rsi->channels,
-				image8->pixels, image8->rowstride,
+			rs_render(&photo->mat, photo->pre_mul, table8, table16, rsi->w, rsi->h, rsi->pixels,
+				rsi->rowstride, image8->pixels, image8->rowstride,
 				rs_cms_get_transform(cms, TRANSFORM_EXPORT));
 			rs_tiff8_save(image8, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
 			rs_image8_free(image8);
@@ -883,16 +897,21 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 		case FILETYPE_TIFF16:
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
 			image16 = rs_image16_new(rsi->w, rsi->h, 3, 3);
-			rs_render16(photo, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, rsi->channels,
-				image16->pixels, image16->rowstride,
+			rs_render16(&photo->mat, photo->pre_mul, table16, rsi->w, rsi->h, rsi->pixels,
+				rsi->rowstride, image16->pixels, image16->rowstride,
 				rs_cms_get_transform(cms, TRANSFORM_EXPORT16));
 			rs_tiff16_save(image16, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
 			rs_image16_free(image16);
 			break;
 	}
-	if (photo->orientation)
-		rs_image16_free(rsi);
+
+	rs_image16_free(rsi);
+
+	photo->exported = TRUE;
+	rs_cache_save(photo);
+
+	/* Set the exported flag */
+	rs_store_set_flags(NULL, photo->filename, NULL, NULL, &photo->exported);
 	return(TRUE);
 }
 
@@ -1317,28 +1336,23 @@ skip_block:
 void
 rs_set_wb_from_pixels(RS_BLOB *rs, gint x, gint y)
 {
-	gint offset, row, col;
+	gint row, col;
+	gushort *pixel;
 	gdouble r=0.0, g=0.0, b=0.0;
 
-	for(row=0; row<3; row++)
+	for(row=-1; row<2; row++)
 	{
-		for(col=0; col<3; col++)
+		for(col=-1; col<2; col++)
 		{
-			offset = (y+row-1)*rs->photo->scaled->rowstride
-				+ (x+col-1)*rs->photo->scaled->pixelsize;
-			r += ((gdouble) rs->photo->scaled->pixels[offset+R])/65535.0;
-			g += ((gdouble) rs->photo->scaled->pixels[offset+G])/65535.0;
-			b += ((gdouble) rs->photo->scaled->pixels[offset+B])/65535.0;
-			if (rs->photo->scaled->channels==4)
-				g += ((gdouble) rs->photo->scaled->pixels[offset+G2])/65535.0;
-				
+			pixel = rs_image16_get_pixel(rs->photo->scaled, x+col, y+row, TRUE);
+			r += pixel[R]/65535.0;
+			g += pixel[G]/65535.0;
+			b += pixel[B]/65535.0;
 		}
 	}
 	r /= 9;
 	g /= 9;
 	b /= 9;
-	if (rs->photo->scaled->channels==4)
-		g /= 2;
 	rs_set_wb_from_color(rs, r, g, b);
 	return;
 }
@@ -1392,13 +1406,8 @@ void
 rs_render_pixel_to_srgb(RS_BLOB *rs, gint x, gint y, guchar *dest)
 {
 	gushort *pixel;
-	if (x>(rs->photo->scaled->w-1))
-		x = rs->photo->scaled->w-1;
-	if (y>(rs->photo->scaled->h-1))
-		y = rs->photo->scaled->h-1;
-	pixel = &rs->photo->scaled->pixels[y*rs->photo->scaled->rowstride
-		+ x*rs->photo->scaled->pixelsize];
-	rs_render_pixel(rs->photo, pixel, dest, rs_cms_get_transform(rs->cms, TRANSFORM_SRGB));
+	pixel = rs_image16_get_pixel(rs->photo->scaled, x, y, TRUE);
+	rs_render_pixel(&rs->photo->mat, rs->photo->pre_mul, rs->previewtable8, rs->previewtable16, pixel, dest, rs_cms_get_transform(rs->cms, TRANSFORM_SRGB));
 	return;
 }
 
@@ -1417,6 +1426,24 @@ rs_apply_settings_from_double(RS_SETTINGS *rss, RS_SETTINGS_DOUBLE *rsd, gint ma
 		SETVAL(rss->warmth,rsd->warmth);
 	if (mask & MASK_TINT)
 		SETVAL(rss->tint,rsd->tint);
+	if (mask & MASK_CURVE) {
+		rs_curve_widget_reset(RS_CURVE_WIDGET(rss->curve));
+
+		if (rsd->curve_nknots>1)
+		{
+			gint i;
+			for(i=0;i<rsd->curve_nknots;i++)
+				rs_curve_widget_add_knot(RS_CURVE_WIDGET(rss->curve), 
+										rsd->curve_knots[i*2+0],
+										rsd->curve_knots[i*2+1]
+										);
+		}
+		else
+		{
+			rs_curve_widget_add_knot(RS_CURVE_WIDGET(rss->curve), 0.0f, 0.0f);
+			rs_curve_widget_add_knot(RS_CURVE_WIDGET(rss->curve), 1.0f, 1.0f);
+		}
+	}
 	return;
 }
 
@@ -1642,6 +1669,25 @@ rs_mark_roi(RS_BLOB *rs, gboolean mark)
 	}
 }
 
+void
+check_install()
+{
+#define TEST_FILE_ACCESS(path) do { if (g_access(path, R_OK)!=0) g_debug("Cannot access %s\n", path);} while (0)
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority1.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority2.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority3.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_deleted.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_exported.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_flip.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_mirror.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_90.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_180.png");
+	TEST_FILE_ACCESS(PACKAGE_DATA_DIR "/pixmaps/rawstudio/transform_270.png");
+
+#undef TEST_FILE_ACCESS
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1660,6 +1706,15 @@ main(int argc, char **argv)
 #endif
 	rs_init_filetypes();
 	gtk_init(&argc, &argv);
+	check_install();
+
+	// initialize overlay icons for thumbnails.
+	icon_priority_1	= gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority1.png", NULL);
+	icon_priority_2 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority2.png", NULL);
+	icon_priority_3 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority3.png", NULL);
+	icon_priority_D = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_deleted.png", NULL);
+	icon_exported = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_exported.png", NULL);
+
 	rs = rs_new();
 	rs->queue->cms = rs->cms = rs_cms_init();
 	gui_init(argc, argv, rs);
@@ -1674,10 +1729,10 @@ rs_shutdown(GtkWidget *dummy1, GdkEvent *dummy2, RS_BLOB *rs)
 	return(TRUE);
 }
 
-/* Inlclude our own g_mkdir_with_parents() in case of old glib.
-Copied almost verbatim from glib-2.10.0/glib/gfileutils.c */
-
 #if !GLIB_CHECK_VERSION(2,8,0)
+
+/* Include our own g_mkdir_with_parents() in case of old glib.
+Copied almost verbatim from glib-2.10.0/glib/gfileutils.c */
 int
 g_mkdir_with_parents (const gchar *pathname,
 		      int          mode)
@@ -1732,4 +1787,49 @@ g_mkdir_with_parents (const gchar *pathname,
 
   return 0;
 }
+
+/* Include our own g_access() in case of old glib.
+Copied almost verbatim from glib-2.12.13/glib/gstdio.h */
+int
+g_access (const gchar *filename,
+	  int          mode)
+{
+#ifdef G_OS_WIN32
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      
+      if (wfilename == NULL)
+	{
+	  return -1;
+	}
+
+      retval = _waccess (wfilename, mode);
+
+      g_free (wfilename);
+
+      return retval;
+    }
+  else
+    {    
+      gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+      int retval;
+
+      if (cp_filename == NULL)
+	{
+	  return -1;
+	}
+
+      retval = access (cp_filename, mode);
+
+      g_free (cp_filename);
+
+      return retval;
+    }
+#else
+  return access (filename, mode);
+#endif
+}
+
 #endif
