@@ -23,12 +23,11 @@
 #include <glib.h>
 #include <lcms.h>
 #include "dcraw_api.h"
-#include "rs-batch.h"
-#include "matrix.h"
 #include "rs-arch.h"
 #include "rs-cms.h"
+#include "rs-curve.h"
 
-#define PITCH(width) ((((width)+31)/32)*32)
+#define PITCH(width) ((((width)+15)/16)*16)
 
 #define SWAP( a, b ) a ^= b ^= a ^= b
 
@@ -54,7 +53,8 @@ enum {
 	MASK_CONTRAST = 8,
 	MASK_WARMTH = 16,
 	MASK_TINT = 32,
-	MASK_ALL = 63
+	MASK_CURVE = 64,
+	MASK_ALL = 127
 };
 
 #define MASK_WB (MASK_WARMTH|MASK_TINT)
@@ -100,6 +100,17 @@ enum {
 #define __deprecated
 #endif
 
+typedef struct _RSStore RSStore;
+
+/* Opaque definition, declared in rs-batch.h */
+typedef struct _RS_QUEUE_ELEMENT RS_QUEUE_ELEMENT;
+typedef struct _RS_QUEUE RS_QUEUE;
+
+typedef struct {double coeff[3][3]; } RS_MATRIX3;
+typedef struct {int coeff[3][3]; } RS_MATRIX3Int;
+typedef struct {double coeff[4][4]; } RS_MATRIX4;
+typedef struct {int coeff[4][4]; } RS_MATRIX4Int;
+
 typedef struct {
 	gint w;
 	gint h;
@@ -109,7 +120,6 @@ typedef struct {
 	guint orientation;
 	guchar *pixels;
 	guint *pixels4;
-	GdkImage *image;
 } RS_IMAGE8;
 
 typedef struct _rs_image16 {
@@ -137,6 +147,8 @@ typedef struct {
 	GtkObject *contrast;
 	GtkObject *warmth;
 	GtkObject *tint;
+	GtkWidget *curve;
+	gfloat curve_samples[65536];
 } RS_SETTINGS;
 
 typedef struct {
@@ -146,6 +158,9 @@ typedef struct {
 	gdouble contrast;
 	gdouble warmth;
 	gdouble tint;
+	gfloat *curve_samples;
+	guint curve_nknots;
+	gfloat *curve_knots;
 } RS_SETTINGS_DOUBLE;
 
 typedef struct _metadata {
@@ -160,6 +175,10 @@ typedef struct _metadata {
 	guint thumbnail_length;
 	guint preview_start;
 	guint preview_length;
+	guint16 preview_planar_config;
+	guint preview_width;
+	guint preview_height;
+	guint16 preview_bits [3];
 	gdouble cam_mul[4];
 	gdouble contrast;
 	gdouble saturation;
@@ -176,19 +195,18 @@ typedef struct _photo {
 	RS_IMAGE16 *input;
 	RS_IMAGE16 *scaled;
 	RS_IMAGE8 *preview;
-	RS_IMAGE8 *mask;
 	RS_SETTINGS_DOUBLE *settings[3];
 	gint current_setting;
 	gint priority;
 	guint orientation;
 	RS_METADATA *metadata;
-	RS_MATRIX4Int mati;
 	RS_MATRIX4 mat;
 	gfloat pre_mul[4];
 	RS_RECT *crop;
 	RS_MATRIX3 affine;
 	RS_MATRIX3 inverse_affine;
 	gdouble angle;
+	gboolean exported;
 } RS_PHOTO;
 
 typedef struct {
@@ -220,6 +238,9 @@ typedef struct {
 	GArray *batch_queue;
 	RS_QUEUE *queue;
 	RS_CMS *cms;
+	guchar previewtable8[65536];
+	gushort previewtable16[65536];
+	RSStore *store;
 } RS_BLOB;
 
 enum {
@@ -242,10 +263,17 @@ typedef struct _rs_filetype {
 	struct _rs_filetype *next;
 } RS_FILETYPE;
 
+extern GdkPixbuf *icon_priority_1;
+extern GdkPixbuf *icon_priority_2;
+extern GdkPixbuf *icon_priority_3;
+extern GdkPixbuf *icon_priority_D;
+extern GdkPixbuf *icon_exported;
+
 void rs_local_cachedir(gboolean new_value);
 void rs_load_gdk(gboolean new_value);
 void update_preview(RS_BLOB *rs, gboolean update_table, gboolean update_scale);
 void update_preview_region(RS_BLOB *rs, RS_RECT *region, gboolean force_render);
+void rs_render_idle_stop(RS_BLOB *rs);
 void rs_reset(RS_BLOB *rs);
 void rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *rs_settings_double);
 void rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SETTINGS *rs_settings);
@@ -255,6 +283,7 @@ RS_PHOTO *rs_photo_new();
 void rs_photo_free(RS_PHOTO *photo);
 gboolean rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype,
 	gint width, gint height, gdouble scale, RS_CMS *cms);
+void rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask);
 RS_METADATA *rs_metadata_new();
 void rs_metadata_free(RS_METADATA *metadata);
 void rs_metadata_normalize_wb(RS_METADATA *meta);
@@ -283,6 +312,7 @@ void rs_mark_roi(RS_BLOB *rs, gboolean mark);
 gboolean rs_shutdown(GtkWidget *dummy1, GdkEvent *dummy2, RS_BLOB *rs);
 #if !GLIB_CHECK_VERSION(2,8,0)
 int g_mkdir_with_parents (const gchar *pathname, int mode);
+int g_access (const gchar *filename, int mode);
 #endif
 
 extern void
