@@ -20,6 +20,7 @@
 #include <glib.h>
 #include <stdio.h>
 #include <gtk/gtk.h>
+#include <config.h>
 #include "rawstudio.h"
 #include "rs-batch.h"
 #include "conf_interface.h"
@@ -29,6 +30,7 @@
 #include "rs-cache.h"
 #include "rs-render.h"
 #include "rs-image.h"
+#include "rs-curve.h"
 
 extern GtkWindow *rawstudio_window;
 
@@ -229,7 +231,7 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 			{
 				if (setting_id == setting_id_temp)
 				{
-					gtk_list_store_remove(GTK_LIST_STORE(queue->list), &iter); /* FIXME: returns false even though the iter is valid and it removes correctly */
+					gtk_list_store_remove(GTK_LIST_STORE(queue->list), &iter);
 					return TRUE;
 				}
 			}
@@ -306,6 +308,11 @@ rs_batch_process(RS_QUEUE *queue)
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
 	GtkWidget *cancel;
 	gboolean abort_render = FALSE;
+	guchar table8[65536];
+	gushort table16[65536];
+	RSCurveWidget *curve = NULL;
+	gint i;
+	gboolean fullscreen = FALSE;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_transient_for(GTK_WINDOW(window), rawstudio_window);
@@ -322,6 +329,12 @@ rs_batch_process(RS_QUEUE *queue)
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), cancel, FALSE, FALSE, 0);
 
+	/* Remember fullscreen state */
+	if (gdk_window_get_state((GTK_WIDGET(rawstudio_window))->window) & GDK_WINDOW_STATE_FULLSCREEN)
+	{
+		fullscreen = TRUE;
+		gtk_window_unfullscreen(rawstudio_window);
+	}
 	gtk_widget_hide(GTK_WIDGET(rawstudio_window));
 	gtk_widget_show_all(window);
 	while (gtk_events_pending()) gtk_main_iteration();
@@ -340,6 +353,8 @@ rs_batch_process(RS_QUEUE *queue)
 			photo = filetype->load(e->filename);
 			if (photo)
 			{
+				gint current_setting;
+				
 				if (filetype->load_meta)
 					filetype->load_meta(e->filename, photo->metadata);
 				filename = g_string_new(queue->directory);
@@ -365,6 +380,9 @@ rs_batch_process(RS_QUEUE *queue)
 
 				parsed_filename = filename_parse(filename->str, photo);
 				rs_cache_load(photo);
+
+				current_setting = photo->current_setting;
+				photo->current_setting = e->setting_id;
 				rs_photo_prepare(photo);
 
 				switch (queue->size_lock)
@@ -382,12 +400,33 @@ rs_batch_process(RS_QUEUE *queue)
 
 				image = rs_image16_transform(photo->input, NULL,
 					NULL, NULL, photo->crop, 200, 200, TRUE, -1.0,
-					photo->angle, photo->orientation);
+					photo->angle, photo->orientation, NULL);
 				if (pixbuf) g_object_unref(pixbuf);
 				pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, image->w, image->h);
-				rs_render(photo, image->w, image->h, image->pixels,
-					image->rowstride, image->channels,
-					gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf),
+
+				if (!curve)
+					curve = RS_CURVE_WIDGET(rs_curve_widget_new()); /* FIXME: free this at some point */
+				rs_curve_widget_reset(curve);
+				if (photo->settings[photo->current_setting]->curve_nknots < 2)
+				{
+					rs_curve_widget_add_knot(curve, 0.0f, 0.0f);
+					rs_curve_widget_add_knot(curve, 1.0f, 1.0f);
+				}
+				else for (i=0;i<photo->settings[photo->current_setting]->curve_nknots;i++)
+				{
+					rs_curve_widget_add_knot(curve,
+						photo->settings[photo->current_setting]->curve_knots[i*2+0],
+						photo->settings[photo->current_setting]->curve_knots[i*2+1]);
+				}
+				photo->settings[photo->current_setting]->curve_samples = g_new(gfloat, 65536);
+				rs_curve_widget_sample(curve, photo->settings[photo->current_setting]->curve_samples, 65536);
+
+				rs_render_previewtable(photo->settings[photo->current_setting]->contrast,
+					photo->settings[photo->current_setting]->curve_samples, table8, table16);
+				
+				rs_render(&photo->mat, photo->pre_mul, table8, table16, image->w, image->h, image->pixels,
+					image->rowstride, gdk_pixbuf_get_pixels(pixbuf),
+					gdk_pixbuf_get_rowstride(pixbuf),
 					rs_cms_get_transform(queue->cms, TRANSFORM_DISPLAY));
 				gtk_image_set_from_pixbuf((GtkImage *) preview, pixbuf);
 				rs_image16_free(image);
@@ -402,6 +441,8 @@ rs_batch_process(RS_QUEUE *queue)
 					width, height, scale, queue->cms);
 				g_free(parsed_filename);
 				g_string_free(filename, TRUE);
+				g_free(photo->settings[photo->current_setting]->curve_samples);
+				photo->current_setting = current_setting;
 				rs_photo_free(photo);
 			}
 			photo = NULL;
@@ -409,6 +450,10 @@ rs_batch_process(RS_QUEUE *queue)
 		rs_batch_remove_element_from_queue(queue, e);
 	}
 	gtk_widget_destroy(window);
+
+	/* Restore fullscreen state if needed */
+	if (fullscreen)	
+		gtk_window_fullscreen(rawstudio_window);
 	gtk_widget_show_all(GTK_WIDGET(rawstudio_window));
 }
 
@@ -587,7 +632,7 @@ chooser_changed(GtkFileChooser *chooser, gpointer user_data)
 {
 	RS_QUEUE *queue = (RS_QUEUE *) user_data;
 	g_free(queue->directory);
-	queue->directory = gtk_file_chooser_get_current_folder(chooser);
+	queue->directory = gtk_file_chooser_get_filename(chooser);
 	rs_conf_set_string(CONF_BATCH_DIRECTORY, queue->directory);
 	return;
 }
@@ -618,7 +663,7 @@ make_batch_options(RS_QUEUE *queue)
 	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(chooser), TRUE);
 	if (g_path_is_absolute(queue->directory))
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), queue->directory);
-	g_signal_connect (chooser, "selection-changed",
+	g_signal_connect (chooser, "current_folder_changed",
 		G_CALLBACK (chooser_changed), queue);
 	gtk_box_pack_start (GTK_BOX (vbox), gui_framed(chooser,
 		_("Output directory:"), GTK_SHADOW_NONE), FALSE, FALSE, 0);
