@@ -17,7 +17,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <gtk/gtk.h>
+#include <glib.h>
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
 #include "rawstudio.h"
@@ -50,7 +50,7 @@ rs_cache_get_name(const gchar *src)
 void
 rs_cache_save(RS_PHOTO *photo)
 {
-	gint id;
+	gint id, i;
 	xmlTextWriterPtr writer;
 	gchar *cachename;
 
@@ -62,6 +62,8 @@ rs_cache_save(RS_PHOTO *photo)
 	xmlTextWriterStartElement(writer, BAD_CAST "rawstudio-cache");
 	xmlTextWriterWriteFormatElement(writer, BAD_CAST "priority", "%d",
 		photo->priority);
+	if (photo->exported)
+		xmlTextWriterWriteFormatElement(writer, BAD_CAST "exported", "yes");
 	xmlTextWriterWriteFormatElement(writer, BAD_CAST "orientation", "%d",
 		photo->orientation);
 	xmlTextWriterWriteFormatElement(writer, BAD_CAST "angle", "%f",
@@ -88,6 +90,16 @@ rs_cache_save(RS_PHOTO *photo)
 			photo->settings[id]->warmth);
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tint", "%f",
 			photo->settings[id]->tint);
+		if (photo->settings[id]->curve_nknots > 0)
+		{
+			xmlTextWriterStartElement(writer, BAD_CAST "curve");
+			xmlTextWriterWriteFormatAttribute(writer, BAD_CAST "num", "%d", photo->settings[id]->curve_nknots);
+			for(i=0;i<photo->settings[id]->curve_nknots;i++)
+				xmlTextWriterWriteFormatElement(writer, BAD_CAST "knot", "%f %f",
+					photo->settings[id]->curve_knots[i*2+0],
+					photo->settings[id]->curve_knots[i*2+1]);
+			xmlTextWriterEndElement(writer);
+		}
 		xmlTextWriterEndElement(writer);
 	}
 	xmlTextWriterEndDocument(writer);
@@ -101,6 +113,7 @@ rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 {
 	xmlChar *val;
 	gdouble *target=NULL;
+	xmlNodePtr curve = NULL;
 	while(cur)
 	{
 		target = NULL;
@@ -116,6 +129,42 @@ rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 			target = &rss->warmth;
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "tint")))
 			target = &rss->tint;
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "curve")))
+		{
+			gchar **vals;
+			gint num;
+			gfloat x,y;
+
+			val = xmlGetProp(cur, BAD_CAST "num");
+			if (val)
+				num = atoi((gchar *) val);
+			else
+				num = 0;
+
+			rss->curve_knots = g_new(gfloat, 2*num);
+			rss->curve_nknots = 0;
+			curve = cur->xmlChildrenNode;
+			while (curve && num)
+			{
+				if ((!xmlStrcmp(curve->name, BAD_CAST "knot")))
+				{
+					val = xmlNodeListGetString(doc, curve->xmlChildrenNode, 1);
+					vals = g_strsplit((gchar *)val, " ", 4);
+					if (vals[0] && vals[1])
+					{
+						x = atof((gchar *) vals[0]);
+						y = atof((gchar *) vals[1]);
+						rss->curve_knots[rss->curve_nknots*2+0] = x;
+						rss->curve_knots[rss->curve_nknots*2+1] = y;
+					}
+					g_strfreev(vals);
+					xmlFree(val);
+				}
+				curve = curve->next;
+				rss->curve_nknots++;
+				if (rss->curve_nknots > num) break;
+			}
+		}
 
 		if (target)
 		{
@@ -139,6 +188,7 @@ rs_cache_load(RS_PHOTO *photo)
 	cachename = rs_cache_get_name(photo->filename);
 	if (!cachename) return(FALSE);
 	if (!g_file_test(cachename, G_FILE_TEST_IS_REGULAR)) return FALSE;
+	photo->exported = FALSE;
 	doc = xmlParseFile(cachename);
 	if(doc==NULL) return(FALSE);
 
@@ -174,6 +224,13 @@ rs_cache_load(RS_PHOTO *photo)
 			photo->angle = g_strtod((gchar *) val, NULL);
 			xmlFree(val);
 		}
+		else if ((!xmlStrcmp(cur->name, BAD_CAST "exported")))
+		{
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			if (g_ascii_strcasecmp((gchar *) val, "yes")==0)
+				photo->exported = TRUE;
+			xmlFree(val);
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "crop")))
 		{
 			gchar **vals;
@@ -209,30 +266,39 @@ rs_cache_load(RS_PHOTO *photo)
 }
 
 void
-rs_cache_load_quick(const gchar *filename, gint *priority)
+rs_cache_load_quick(const gchar *filename, gint *priority, gboolean *exported)
 {
 	xmlDocPtr doc;
 	xmlNodePtr cur;
 	xmlChar *val;
 	gchar *cachename;
 
+	if (!filename) return;
 	cachename = rs_cache_get_name(filename);
 	if (!cachename) return;
 	if (!g_file_test(cachename, G_FILE_TEST_IS_REGULAR)) return;
 	doc = xmlParseFile(cachename);
 	if(doc==NULL) return;
+	if (exported) *exported = FALSE;
 
 	cur = xmlDocGetRootElement(doc);
 
 	cur = cur->xmlChildrenNode;
 	while(cur)
 	{
-		if ((!xmlStrcmp(cur->name, BAD_CAST "priority")))
+		if (priority && (!xmlStrcmp(cur->name, BAD_CAST "priority")))
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
 			*priority = atoi((gchar *) val);
 			xmlFree(val);
 		}
+		if (exported && (!xmlStrcmp(cur->name, BAD_CAST "exported")))
+		{
+			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
+			if (g_ascii_strcasecmp((gchar *) val, "yes")==0)
+				*exported = TRUE;
+			xmlFree(val);
+		}
 		cur = cur->next;
 	}
 	
@@ -240,3 +306,59 @@ rs_cache_load_quick(const gchar *filename, gint *priority)
 	g_free(cachename);
 	return;
 }
+
+void
+rs_cache_save_flags(const gchar *filename, const guint *priority, const gboolean *exported)
+{
+	RS_PHOTO *photo;
+
+	g_assert(filename != NULL);
+
+	if (!(priority || exported)) return;
+
+	/* Aquire a "fake" RS_PHOTO */
+	photo = rs_photo_new();
+	photo->active = TRUE;
+	photo->filename = (gchar *) filename;
+
+	if (rs_cache_load(photo))
+	{
+		/* If we got a cache file, save as normal */
+		if (priority)
+			photo->priority = *priority;
+		if (exported)
+			photo->exported = *exported;
+		rs_cache_save(photo);
+	}
+	else
+	{
+		/* If we're creating a new file, only save what we know */
+		xmlTextWriterPtr writer;
+		gchar *cachename = rs_cache_get_name(photo->filename);
+
+		if (cachename)
+		{
+			writer = xmlNewTextWriterFilename(cachename, 0); /* fixme, check for errors */
+			g_free(cachename);
+
+			xmlTextWriterStartDocument(writer, NULL, "ISO-8859-1", NULL);
+			xmlTextWriterStartElement(writer, BAD_CAST "rawstudio-cache");
+
+			if (priority)
+				xmlTextWriterWriteFormatElement(writer, BAD_CAST "priority", "%d",
+					*priority);
+
+			if (exported && *exported)
+				xmlTextWriterWriteFormatElement(writer, BAD_CAST "exported", "yes");
+
+			xmlTextWriterEndDocument(writer);
+			xmlFreeTextWriter(writer);
+		}
+	}
+
+	/* Free the photo */
+	photo->filename = NULL;
+	rs_photo_free(photo);
+
+	return;
+}
