@@ -26,7 +26,6 @@
 #include <math.h> /* floor() */
 #include "color.h"
 #include "matrix.h"
-#include "rs-batch.h"
 #include "rawstudio.h"
 #include "rs-image.h"
 
@@ -35,6 +34,7 @@ static void rs_image16_mirror(RS_IMAGE16 *rsi);
 static void rs_image16_flip(RS_IMAGE16 *rsi);
 inline static void rs_image16_nearest(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y);
 inline static void rs_image16_bilinear(RS_IMAGE16 *in, gushort *out, gdouble x, gdouble y);
+static void rs_image8_realloc(RS_IMAGE8 *rsi, const guint width, const guint height, const guint channels, const guint pixelsize);
 
 void
 rs_image16_orientation(RS_IMAGE16 *rsi, const gint orientation)
@@ -271,9 +271,25 @@ rs_image16_transform_getwh(RS_IMAGE16 *in, RS_RECT *crop, gdouble angle, gint or
 	return;
 }
 
+/**
+ * Transforms an RS_IMAGE16
+ * @param in An input image
+ * @param out An output image or NULL
+ * @param affine Will be set to forward affine matrix if not NULL.
+ * @param inverse_affine Will be set to inverse affine matrix if not NULL.
+ * @param crop Crop to apply or NULL
+ * @param width Output width or -1
+ * @param height Output height or -1
+ * @param keep_aspect if set to TRUE aspect will be locked
+ * @param scale How much to scale the image (0.01 - 2.0)
+ * @param angle Rotation angle in degrees
+ * @param orientation The orientation
+ * @param actual_scale The resulting scale or NULL
+ * @return A new RS_IMAGE16 or out
+ */
 RS_IMAGE16 *
 rs_image16_transform(RS_IMAGE16 *in, RS_IMAGE16 *out, RS_MATRIX3 *affine, RS_MATRIX3 *inverse_affine,
-	RS_RECT *crop, gint width, gint height, gboolean keep_aspect, gdouble scale, gdouble angle, gint orientation)
+	RS_RECT *crop, gint width, gint height, gboolean keep_aspect, gdouble scale, gdouble angle, gint orientation, gdouble *actual_scale)
 {
 	RS_MATRIX3 mat;
 	gdouble xscale=1.0, yscale=1.0;
@@ -340,6 +356,10 @@ rs_image16_transform(RS_IMAGE16 *in, RS_IMAGE16 *out, RS_MATRIX3 *affine, RS_MAT
 	/* scale */
 	matrix3_affine_scale(&mat, xscale, yscale);
 
+	/* Write back the actual scaling if requested */
+	if (actual_scale)
+		*actual_scale = (xscale+yscale)/2.0;
+
 	/* apply scaling to our previously calculated width and height */
 	w *= xscale;
 	h *= yscale;
@@ -535,7 +555,7 @@ rs_image16_new(const guint width, const guint height, const guint channels, cons
 	rsi->channels = channels;
 	rsi->pixelsize = pixelsize;
 	ORIENTATION_RESET(rsi->orientation);
-	rsi->pixels = (gushort *) g_malloc(sizeof(gushort)*rsi->h*rsi->rowstride);
+	rsi->pixels = g_new0(gushort, rsi->h*rsi->rowstride);
 	return(rsi);
 }
 
@@ -556,30 +576,16 @@ RS_IMAGE8 *
 rs_image8_new(const guint width, const guint height, const guint channels, const guint pixelsize)
 {
 	RS_IMAGE8 *rsi;
-	GdkVisual *vis;
 
 	rsi = (RS_IMAGE8 *) g_malloc(sizeof(RS_IMAGE8));
-	rsi->image = NULL;
 	rsi->w = width;
 	rsi->h = height;
 	ORIENTATION_RESET(rsi->orientation);
-	if ((channels==3) && (pixelsize==4))
-	{
-		vis = gdk_visual_get_system();
-		rsi->image = gdk_image_new(GDK_IMAGE_FASTEST, vis, width, height);
-		rsi->pixels = rsi->image->mem;
-		rsi->rowstride = rsi->image->bpl;
-		rsi->channels = channels;
-		rsi->pixelsize = rsi->image->bpp;
-	}
-	else
-	{
-		rsi->image = NULL;
-		rsi->rowstride = PITCH(width) * pixelsize;
-		rsi->pixels = (guchar *) g_malloc(sizeof(guchar)*rsi->h*rsi->rowstride);
-		rsi->channels = channels;
-		rsi->pixelsize = pixelsize;
-	}
+	rsi->rowstride = PITCH(width) * pixelsize;
+	rsi->pixels = g_new0(guchar, rsi->h*rsi->rowstride);
+	rsi->channels = channels;
+	rsi->pixelsize = pixelsize;
+
 	return(rsi);
 }
 
@@ -588,15 +594,111 @@ rs_image8_free(RS_IMAGE8 *rsi)
 {
 	if (rsi!=NULL)
 	{
-		if (rsi->image)
-			g_object_unref(rsi->image);
-		else
-			g_free(rsi->pixels);
+		g_free(rsi->pixels);
 		g_free(rsi);
 	}
 	return;
 }
 
+static void
+rs_image8_realloc(RS_IMAGE8 *rsi, const guint width, const guint height, const guint channels, const guint pixelsize)
+{
+	if (!rsi) return;
+
+	/* Do we actually differ? */
+	if ((rsi->w != width) || (rsi->h != height) || (rsi->channels != channels) || (rsi->pixelsize != pixelsize))
+	{
+		/* Free the old pixels */
+		g_free(rsi->pixels);
+
+		/* Fill in new values */
+		rsi->w = width;
+		rsi->h = height;
+		rsi->rowstride = PITCH(width) * pixelsize;
+		rsi->pixels = g_new0(guchar, rsi->h*rsi->rowstride);
+		rsi->channels = channels;
+		rsi->pixelsize = pixelsize;
+	}
+}
+
+/**
+ * Renders an exposure map on top of an RS_IMAGE8 with 3 channels
+ * @param image A RS_IMAGE8
+ * @param only_row A single row to render or -1 to render all
+ */
+void
+rs_image8_render_exposure_mask(RS_IMAGE8 *image, gint only_row)
+{
+	gint row, col;
+	gint start;
+	gint stop;
+
+	g_assert(image != NULL);
+	g_assert(image->channels == 3);
+
+	if ((only_row > -1) && (only_row < image->h))
+	{
+		start = only_row;
+		stop = only_row + 1;
+	}
+	else
+	{
+		start = 0;
+		stop = image->h;
+	}
+
+	for(row=start;row<stop;row++)
+	{
+		/* Get start pixel of row */
+		guchar *pixel = GET_PIXEL(image, 0, row);
+
+		for(col=0;col<image->w;col++)
+		{
+			/* Catch pixels overexposed and color them red */
+			if ((pixel[R]==0xFF) || (pixel[G]==0xFF) || (pixel[B]==0xFF))
+			{
+				*pixel++ = 0xFF;
+				*pixel++ = 0x00;
+				*pixel++ = 0x00;
+			}
+			/* Color underexposed pixels blue */
+			else if ((pixel[R]<2) && (pixel[G]<2) && (pixel[B]<2))
+			{
+				*pixel++ = 0x00;
+				*pixel++ = 0x00;
+				*pixel++ = 0xFF;
+			}
+			else
+				pixel += 3;
+		}
+	}
+}
+
+/**
+ * Renders a shaded image
+ * @param in The input image
+ * @param out The output image or NULL
+ * @return The shaded image
+ */
+RS_IMAGE8 *
+rs_image8_render_shaded(RS_IMAGE8 *in, RS_IMAGE8 *out)
+{
+	gint size;
+
+	if (!in) return NULL;
+
+	if (!out)
+		out = rs_image8_new(in->w, in->h, in->channels, in->pixelsize);
+	else
+		rs_image8_realloc(out, in->w, in->h, in->channels, in->pixelsize);
+
+	size = in->h * in->rowstride;
+	while(size--)
+		out->pixels[size] = ((in->pixels[size]+63)*3)>>3; /* Magic shade formula :) */
+
+	return out;
+}
+
 RS_IMAGE16 *
 rs_image16_copy(RS_IMAGE16 *in)
 {
@@ -606,6 +708,40 @@ rs_image16_copy(RS_IMAGE16 *in)
 	return(out);
 }
 
+/**
+ * Returns a single pixel from a RS_IMAGE16
+ * @param image A RS_IMAGE16
+ * @param x X coordinate (column)
+ * @param y Y coordinate (row)
+ * @param extend_edges Tries to extend edges beyond image borders if TRUE
+ */
+inline gushort *
+rs_image16_get_pixel(RS_IMAGE16 *image, gint x, gint y, gboolean extend_edges)
+{
+	gushort *pixel = NULL;
+
+	if (image)
+	{
+		if (extend_edges)
+		{
+			if (x >= image->w)
+				x = image->w-1;
+			else if (x < 0)
+				x = 0;
+			if (y >= image->h)
+				y = image->h-1;
+			else if (y < 0)
+				y = 0;
+		}
+
+		/* Return pixel if inside image */
+		if ((x>=0) && (y>=0) && (x<image->w) && (y<image->h))
+			pixel = &image->pixels[y*image->rowstride + x*image->pixelsize];
+	}
+
+	return pixel;
+}
+
 gboolean
 rs_image16_8_cmp_size(RS_IMAGE16 *a, RS_IMAGE8 *b)
 {
